@@ -586,17 +586,10 @@ SEXP ps__boot_time() {
  * Return the number of logical, active CPUs. Return 0 if undetermined.
  * See discussion at: https://bugs.python.org/issue33166#msg314631
  */
-#if (_WIN32_WINNT < 0x0601)  // < Windows 7 (namely Vista and XP)
 
-SEXP ps__cpu_count_logical() {
-  return ScalarInteger(NA_INTEGER);
-}
-
-SEXP ps__cpu_count_physical() {
-  return ScalarInteger(NA_INTEGER);
-}
-
-#else  // Windows >= 7
+#ifndef ALL_PROCESSOR_GROUPS
+#define ALL_PROCESSOR_GROUPS 0xffff
+#endif
 
 unsigned int ps__get_num_cpus(int fail_on_err) {
   unsigned int ncpus = 0;
@@ -721,7 +714,6 @@ SEXP ps__cpu_count_physical() {
     return ScalarInteger(NA_INTEGER);
   }
 }
-#endif
 
 SEXP ps__kill_if_env(SEXP marker, SEXP after, SEXP pid, SEXP sig) {
   const char *cmarker = CHAR(STRING_ELT(marker, 0));
@@ -870,6 +862,36 @@ SEXP psll_open_files(SEXP p) {
   return result;
 }
 
+SEXP psll_dlls(SEXP p) {
+  ps_handle_t *handle = R_ExternalPtrAddr(p);
+  HANDLE processHandle = NULL;
+  DWORD access = PROCESS_QUERY_INFORMATION | PROCESS_VM_READ;
+  SEXP result;
+
+  if (!handle) error("Process pointer cleaned up already");
+
+  processHandle = ps__handle_from_pid_waccess(handle->pid, access);
+  if (processHandle == NULL) {
+    PS__CHECK_HANDLE(handle);
+    ps__set_error_from_windows_error(0);
+    ps__throw_error();
+  }
+
+  PROTECT(result = ps__get_modules(processHandle));
+
+  CloseHandle(processHandle);
+
+  PS__CHECK_HANDLE(handle);
+
+  if (isNull(result)) {
+    ps__set_error_from_windows_error(0);
+    ps__throw_error();
+  }
+
+  UNPROTECT(1);
+  return result;
+}
+
 SEXP psll_interrupt(SEXP p, SEXP ctrlc, SEXP interrupt_path) {
   ps_handle_t *handle = R_ExternalPtrAddr(p);
   const char *cinterrupt_path = CHAR(STRING_ELT(interrupt_path, 0));
@@ -1058,3 +1080,319 @@ SEXP ps__users() {
   ps__throw_error();
   return R_NilValue;
 }
+
+SEXP ps__tty_size() {
+  CONSOLE_SCREEN_BUFFER_INFO info;
+
+  BOOL ok = GetConsoleScreenBufferInfo(
+    GetStdHandle(STD_OUTPUT_HANDLE),
+    &info
+  );
+
+  if (!ok) {
+    ps__set_error_from_windows_error(0);
+    ps__throw_error();
+  }
+
+  SEXP result = Rf_allocVector(INTSXP, 2);
+  INTEGER(result)[0] = info.srWindow.Right  - info.srWindow.Left + 1;
+  INTEGER(result)[1] = info.srWindow.Bottom - info.srWindow.Top + 1;
+
+  return result;
+}
+
+static char *ps__get_drive_type(int type) {
+  switch (type) {
+  case DRIVE_FIXED:
+    return "fixed";
+  case DRIVE_CDROM:
+    return "cdrom";
+  case DRIVE_REMOVABLE:
+    return "removable";
+  case DRIVE_UNKNOWN:
+    return "unknown";
+  case DRIVE_NO_ROOT_DIR:
+    return "unmounted";
+  case DRIVE_REMOTE:
+    return "remote";
+  case DRIVE_RAMDISK:
+    return "ramdisk";
+  default:
+    return "?";
+  }
+}
+
+
+
+SEXP ps__disk_partitions(SEXP rall) {
+  DWORD num_bytes;
+  char drive_strings[255];
+  char *drive_letter = drive_strings;
+  char mp_buf[MAX_PATH];
+  char mp_path[MAX_PATH];
+  int all = LOGICAL(rall)[0];
+  int type;
+  int ret;
+  unsigned int old_mode = 0;
+  char opts[20];
+  HANDLE mp_h;
+  BOOL mp_flag= TRUE;
+  LPTSTR fs_type[MAX_PATH + 1] = { 0 };
+  DWORD pflags = 0;
+
+  SEXP result;
+  PROTECT_INDEX pidx;
+  int len = 10, num = -1;
+  PROTECT_WITH_INDEX(result = allocVector(VECSXP, len), &pidx);
+
+  // avoid to visualize a message box in case something goes wrong
+  // see https://github.com/giampaolo/psutil/issues/264
+  old_mode = SetErrorMode(SEM_FAILCRITICALERRORS);
+
+  num_bytes = GetLogicalDriveStrings(254, drive_letter);
+
+  if (num_bytes == 0) {
+    ps__set_error_from_errno(0);
+    goto error;
+  }
+
+  while (*drive_letter != 0) {
+    opts[0] = 0;
+    fs_type[0] = 0;
+
+    type = GetDriveType(drive_letter);
+
+    // by default we only show hard drives and cd-roms
+    if (all == 0) {
+      if ((type == DRIVE_UNKNOWN) ||
+          (type == DRIVE_NO_ROOT_DIR) ||
+          (type == DRIVE_REMOTE) ||
+          (type == DRIVE_RAMDISK)) {
+        goto next;
+      }
+      // floppy disk: skip it by default as it introduces a
+      // considerable slowdown.
+      if ((type == DRIVE_REMOVABLE) &&
+          (strcmp(drive_letter, "A:\\")  == 0)) {
+        goto next;
+      }
+    }
+
+    ret = GetVolumeInformation(
+      (LPCTSTR)drive_letter, NULL, _ARRAYSIZE(drive_letter),
+      NULL, NULL, &pflags, (LPTSTR)fs_type, _ARRAYSIZE(fs_type));
+    if (ret == 0) {
+      // We might get here in case of a floppy hard drive, in
+      // which case the error is (21, "device not ready").
+      // Let's pretend it didn't happen as we already have
+      // the drive name and type ('removable').
+      strcat_s(opts, _countof(opts), "");
+      SetLastError(0);
+    } else {
+      if (pflags & FILE_READ_ONLY_VOLUME) {
+        strcat_s(opts, _countof(opts), "ro");
+      } else {
+        strcat_s(opts, _countof(opts), "rw");
+      }
+      if (pflags & FILE_VOLUME_IS_COMPRESSED) {
+        strcat_s(opts, _countof(opts), ",compressed");
+      }
+
+      // Check for mount points on this volume and add/get info
+      // (checks first to know if we can even have mount points)
+      if (pflags & FILE_SUPPORTS_REPARSE_POINTS) {
+        mp_h = FindFirstVolumeMountPoint(
+          drive_letter, mp_buf, MAX_PATH);
+        if (mp_h != INVALID_HANDLE_VALUE) {
+          while (mp_flag) {
+
+            // Append full mount path with drive letter
+            strcpy_s(mp_path, _countof(mp_path), drive_letter);
+            strcat_s(mp_path, _countof(mp_path), mp_buf);
+
+            SetErrorMode(old_mode);
+            // TODO: should call FindVolumeMountPointClose on error
+            if (++num == len) {
+              len *= 2;
+              REPROTECT(result = Rf_lengthgets(result, len), pidx);
+            }
+            SET_VECTOR_ELT(
+              result, num,
+              ps__build_list(
+                "ssss", drive_letter, mp_path, fs_type, opts));
+            SetErrorMode(SEM_FAILCRITICALERRORS);
+
+            // Continue looking for more mount points
+            mp_flag = FindNextVolumeMountPoint(
+              mp_h, mp_buf, MAX_PATH);
+          }
+          FindVolumeMountPointClose(mp_h);
+        }
+      }
+    }
+
+    if (strlen(opts) > 0) {
+      strcat_s(opts, _countof(opts), ",");
+    }
+    strcat_s(opts, _countof(opts), ps__get_drive_type(type));
+
+    SetErrorMode(old_mode);
+
+    if (++num == len) {
+      len *= 2;
+      REPROTECT(result = Rf_lengthgets(result, len), pidx);
+    }
+    SET_VECTOR_ELT(
+      result, num,
+      ps__build_list(
+        "ssss", drive_letter, drive_letter, fs_type, opts));
+
+    next:
+      drive_letter = strchr(drive_letter, 0) + 1;
+  }
+
+  UNPROTECT(1);
+  return result;
+
+  error:
+    SetErrorMode(old_mode);
+    return R_NilValue;
+}
+
+SEXP ps__disk_usage(SEXP paths) {
+  BOOL retval;
+  ULARGE_INTEGER freeuser, total, free;
+  int i, n = Rf_length(paths);
+  SEXP result = PROTECT(allocVector(VECSXP, n));
+
+  for (i = 0; i < n; i++) {
+    const char *path = CHAR(STRING_ELT(paths, i));
+    wchar_t *wpath;
+    int iret = ps__utf8_to_utf16(path, &wpath);
+    if (iret) goto error;
+    retval = GetDiskFreeSpaceExW(wpath, &freeuser, &total, &free);
+    if (!retval) {
+      ps__set_error_from_windows_error(0);
+      goto error;
+    }
+    SET_VECTOR_ELT(
+      result, i,
+      allocVector(REALSXP, 3));
+    REAL(VECTOR_ELT(result, i))[0] = (double) total.QuadPart;
+    REAL(VECTOR_ELT(result, i))[1] = (double) free.QuadPart;
+    REAL(VECTOR_ELT(result, i))[2] = (double) freeuser.QuadPart;
+  }
+
+  UNPROTECT(1);
+  return result;
+
+error:
+  ps__throw_error();
+  return R_NilValue;
+}
+
+SEXP ps__system_memory() {
+  MEMORYSTATUSEX memInfo;
+  memInfo.dwLength = sizeof(MEMORYSTATUSEX);
+
+  if (! GlobalMemoryStatusEx(&memInfo)) {
+    ps__set_error_from_windows_error(0);
+    ps__throw_error();
+  }
+
+  return ps__build_named_list(
+    "dddddd",
+    "total",      (double) memInfo.ullTotalPhys,
+    "avail",      (double) memInfo.ullAvailPhys,
+    "totpagef",   (double) memInfo.ullTotalPageFile,
+    "availpagef", (double) memInfo.ullAvailPageFile,
+    "totvirt",    (double) memInfo.ullTotalVirtual,
+    "freevirt",   (double) memInfo.ullAvailVirtual);
+}
+
+SEXP ps__system_swap() {
+  // We use ps__system_memory instead
+  ps__throw_error();
+  return R_NilValue;
+}
+
+SEXP psll_get_nice(SEXP p) {
+  ps_handle_t *handle = R_ExternalPtrAddr(p);
+  HANDLE hProcess = NULL;
+  DWORD priority;
+
+  if (!handle) error("Process pointer cleaned up already");
+
+  hProcess = ps__handle_from_pid(handle->pid);
+  if (!hProcess) goto error;
+
+  priority = GetPriorityClass(hProcess);
+  if (priority == 0) {
+    ps__set_error_from_windows_error(0);
+    goto error;
+  }
+
+  CloseHandle(hProcess);
+
+  switch(priority) {
+  case REALTIME_PRIORITY_CLASS:
+    return ScalarInteger(1);
+  case HIGH_PRIORITY_CLASS:
+    return ScalarInteger(2);
+  case ABOVE_NORMAL_PRIORITY_CLASS:
+    return ScalarInteger(3);
+  case NORMAL_PRIORITY_CLASS:
+    return ScalarInteger(4);
+  case IDLE_PRIORITY_CLASS:
+    return ScalarInteger(5);
+  case BELOW_NORMAL_PRIORITY_CLASS:
+    return ScalarInteger(6);
+  default:
+    error("Internal error, invalid priority class");
+  }
+
+error:
+  if (hProcess) CloseHandle(hProcess);
+  ps__throw_error();
+  return R_NilValue;
+}
+
+SEXP psll_set_nice(SEXP p, SEXP value) {
+  ps_handle_t *handle = R_ExternalPtrAddr(p);
+  HANDLE hProcess = NULL;
+  DWORD priority = INTEGER(value)[0];
+  DWORD classes[] = {
+    REALTIME_PRIORITY_CLASS,
+    HIGH_PRIORITY_CLASS,
+    ABOVE_NORMAL_PRIORITY_CLASS,
+    NORMAL_PRIORITY_CLASS,
+    IDLE_PRIORITY_CLASS,
+    BELOW_NORMAL_PRIORITY_CLASS
+  };
+  DWORD access = PROCESS_QUERY_INFORMATION | PROCESS_SET_INFORMATION;
+
+  if (!handle) error("Process pointer cleaned up already");
+
+  hProcess = ps__handle_from_pid_waccess(handle->pid, access);
+  if (!hProcess) {
+    PS__CHECK_HANDLE(handle);
+    ps__set_error_from_windows_error(0);
+    goto error;
+  }
+
+  BOOL ret = SetPriorityClass(hProcess, classes[priority - 1]);
+  if (!ret) {
+    ps__set_error_from_windows_error(0);
+    goto error;
+  }
+
+  CloseHandle(hProcess);
+
+  return R_NilValue;
+
+error:
+  if (hProcess) CloseHandle(hProcess);
+  ps__throw_error();
+  return R_NilValue;
+
+}
