@@ -38,6 +38,7 @@ SEXP psp__stat_st_rdev(SEXP x) { return ps__dummy("psp__stat_st_rdev"); }
 #ifdef PS__POSIX
 #ifndef PS__WINDOWS
 SEXP psw__realpath(SEXP x) { return ps__dummy("psw__realpath"); }
+SEXP psll_dlls(SEXP x) { return ps__dummy("psll_dlls"); }
 #endif
 #endif
 
@@ -55,7 +56,12 @@ SEXP ps__pids() { return ps__dummy("ps_pids"); }
 SEXP ps__boot_time() { return ps__dummy("ps_boot_time"); }
 SEXP ps__cpu_count_logical()  { return ps__dummy("ps_cpu_count"); }
 SEXP ps__cpu_count_physical() { return ps__dummy("ps_cpu_count"); }
-SEXP ps__users() { ps__users("ps_users"); }
+SEXP ps__users() { ps__dummy("ps_users"); }
+SEXP ps__tty_size() { ps__dummy("ps_tty_size"); }
+SEXP ps__disk_partitions(SEXP x) { ps__dummy("ps_disk_partitions"); }
+SEXP ps__disk_usage() { ps__dummy("ps_disk_usage"); }
+SEXP ps__system_memory() { ps__dummy("ps_system_memory"); }
+SEXP ps__system_swap() { ps__dummy("ps_system_swap"); }
 
 SEXP psll_handle(SEXP x, SEXP y) { return ps__dummy("ps_handle"); }
 SEXP psll_format(SEXP x) { return ps__dummy("ps_format"); }
@@ -84,6 +90,8 @@ SEXP psll_num_fds(SEXP x) { return ps__dummy("ps_num_fds"); }
 SEXP psll_open_files(SEXP x) { return ps__dummy("ps_open_files"); }
 SEXP psll_interrupt(SEXP x, SEXP y, SEXP z) { return ps__dummy("ps_interrupt"); }
 SEXP psll_connections(SEXP x) { return ps__dummy("ps_connections"); }
+SEXP psll_get_nice(SEXP x) { return ps__dummy("ps_get_nice"); }
+SEXP psll_set_nice(SEXP x, SEXP y) { return ps__dummy("ps_set_nice"); }
 
 SEXP ps__init(SEXP x, SEXP y) { return R_NilValue; /* this needs to run to load package */ }
 SEXP ps__kill_if_env(SEXP x, SEXP y, SEXP z, SEXP a) { return ps__dummy("ps__kill_if_env"); }
@@ -95,6 +103,7 @@ SEXP psp__zombie() { return ps__dummy("psp__zombie"); }
 SEXP psp__waitpid(SEXP x) { return ps__dummy("psp__waitpid"); }
 
 SEXP psw__realpath(SEXP x) { return ps__dummy("psw__realpath"); }
+SEXP psll_dlls(SEXP x) { return ps__dummy("psll_dlls"); }
 
 SEXP ps__inet_ntop(SEXP x, SEXP y) { return ps__dummy("ps__inet_ntop"); }
 #endif
